from meiga.error import Error
from meiga.result import Result


class Success(Result):
    def __init__(self, value=True) -> None:
        Result.__init__(self, success=value)


class Failure(Result):
    def __init__(self, error=Error()) -> None:
        Result.__init__(self, failure=error)


isSuccess = Success()
isFailure = Failure()
NotImplementedMethodError = isFailure

BoolResult = Result[bool, Error]



from meiga.alias import Failure
from meiga.decorators.unexpected_decoration_order_error import (
    UnexpectedDecorationOrderError,
)
from meiga.error import Error
from meiga.on_failure_exception import OnFailureException


def meiga(func):
    def _meiga(*args, **kwargs):
        try:
            if isinstance(func, staticmethod):
                return Failure(UnexpectedDecorationOrderError())
            elif isinstance(func, classmethod):
                return Failure(UnexpectedDecorationOrderError())
            else:
                return func(*args, **kwargs)
        except OnFailureException as exc:
            return exc.result
        except Error as error:
            return Failure(error)

    return _meiga



# coding=utf-8
# Copyright (C) 2021+ Alice, Vigo, Spain

"""Public API of Alice meiga package"""

# Modules
from . import decorators

modules = ["decorators"]

# Classes
from .result import Result

classes = ["Result"]

# Errors
from .error import Error

errors = ["Error"]

# Alias
from .alias import (
    BoolResult,
    Failure,
    NotImplementedMethodError,
    Success,
    isFailure,
    isSuccess,
)

alias = [
    "Success",
    "isSuccess",
    "Failure",
    "isFailure",
    "NotImplementedMethodError",
    "BoolResult",
]

__all__ = modules + classes + errors + alias



class Error(Exception):
    def __init__(self):
        self.message = None

    def __eq__(self, other) -> bool:
        if isinstance(other, self.__class__):
            return True
        return False

    def __repr__(self) -> str:
        suffix = ""
        if self.message:
            suffix = f": {self.message}"

        return f"{self.__class__.__name__}{suffix}"



import os

from setuptools import setup

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
PACKAGE_NAME = "meiga"
VERSION = open("meiga/VERSION", "r").read()

# The text of the README file
with open(os.path.join(CURRENT_DIR, "README.md")) as fid:
    README = fid.read()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description="A simple, typed and monad-based Result type for Python",
    long_description=README,
    long_description_content_type="text/markdown",
    keywords=["Result", "Monad", "Typed", "Typing"],
    url="https://github.com/alice-biometrics/meiga",
    author="Alice Biometrics",
    author_email="support@alicebiometrics.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=["meiga", "meiga/assertions", "meiga/decorators"],
    include_package_data=True,
    zip_safe=False,
)
